import java.util.LinkedList;

public class AdjListGraph<VL, EL> implements Graph<VL, EL> {

	class Vertex {
		VL label;
		LinkedList<Edge> outgoing;

		Vertex(VL label) {
			this.label= label;
			outgoing= new LinkedList<>();
		}
	}

	class Edge {
		EL label;
		VL to;

		Edge(EL label, VL to) {
			this.label= label;
			this.to= to;
		}
	}

	LinkedList<Vertex> vertices;

	public AdjListGraph() {
		vertices= new LinkedList<>();
	}

	@Override
	public int numVertices() {
		return vertices.size();
	}

	@Override
	public boolean hasVertex(VL v) {
		for (Vertex u : vertices) {
			if (u.label.equals(v)) return true;
		}
		return false;
	}

	@Override
	public void addVertex(VL v) {
		if (hasVertex(v)) throw new VertexAlreadyInGraph(v.toString());
		Vertex u= new Vertex(v);
		vertices.addFirst(u);
	}

	@Override
	public int numEdges() {
		int e= 0;
		for (Vertex v : vertices) {
			e+= v.outgoing.size();
		}
		return e;
	}

	private void checkInGraph(VL v) {
		if (!hasVertex(v)) throw new VertexNotInGraph(v.toString());
	}

	@Override
	public EL edgeLabel(VL v1, VL v2) {
		for (Vertex v : vertices) {
			if (v.label.equals(v1)) {
				// v1 is in the graph
				for (Edge e : v.outgoing) {
					if (e.to.equals(v2)) {
						// v2 is adjacent to v1
						return e.label;
					}
				}
				// v2 is not adjacent to v1
				throw new EdgeNotInGraph(v1.toString(), v2.toString());
			}
		}
		throw new EdgeNotInGraph(v1.toString(), v2.toString());
	}

	@Override
	public boolean hasEdge(VL v1, VL v2) {
		try {
			edgeLabel(v1, v2);
		} catch (EdgeNotInGraph e) {
			return false;
		}
		return true;
	}

	@Override
	public void addEdge(VL v1, EL e, VL v2) {
		for (Vertex v : vertices) {
			if (v.label.equals(v1)) {
				// v1 is in the graph
				for (Edge f : v.outgoing) {
					if (f.to.equals(v2)) {
						// v2 is already adjacent to v1
						throw new EdgeAlreadyInGraph(v1.toString(),
							f.label.toString(), v2.toString());
					}
				}
				// v2 is not already adjacent to v1; make it so
				checkInGraph(v2);
				v.outgoing.add(new Edge(e, v2));
				return;
			}
		}
		throw new VertexNotInGraph(v1.toString());
	}

	@Override
	public String toString() {
		String s= "";
		for (Vertex v : vertices) {
			s+= v.label + ": ";
			for (Edge e : v.outgoing) {
				s+= "[" + e.label + "] " + e.to + "; ";
			}
			s+= "\n";
		}
		return s;
	}

	public static void main(String[] args) {
		// construct and print example graph from lecture slide on "Labels"
		Graph<String, Integer> g= new AdjListGraph<>();
		g.addVertex("A");
		g.addVertex("B");
		g.addVertex("C");
		g.addVertex("D");
		g.addVertex("E");
		g.addEdge("A", 5, "C");
		g.addEdge("B", 2, "A");
		g.addEdge("C", -3, "D");
		g.addEdge("D", 1, "C");
		System.out.println(g);
	}

}
